@@ -5,17 +5,25 @@
  */
 
 #include "mediaplayer2player.h"
-#include "mpvitem.h"
 
+#include "application.h"
+#include "mpvitem.h"
+#include "videosettings.h"
+#include "worker.h"
+
+#include <QBuffer>
 #include <QDBusConnection>
 #include <QDBusMessage>
 #include <QDBusObjectPath>
+#include <QImage>
+
+#include <KFileMetaData/ExtractorCollection>
+#include <KFileMetaData/SimpleExtractionResult>
 
 MediaPlayer2Player::MediaPlayer2Player(QObject *parent)
     : QDBusAbstractAdaptor(parent)
 {
-    connect(this, &MediaPlayer2Player::mpvChanged,
-            this, &MediaPlayer2Player::setupConnections);
+    connect(this, &MediaPlayer2Player::mpvChanged, this, &MediaPlayer2Player::setupConnections);
 }
 
 void MediaPlayer2Player::setupConnections()
@@ -90,13 +98,13 @@ void MediaPlayer2Player::Play()
 
 void MediaPlayer2Player::Seek(qlonglong offset)
 {
-    Q_EMIT seek(offset/1000/1000);
+    Q_EMIT seek(offset / 1000 / 1000);
 }
 
 void MediaPlayer2Player::SetPosition(const QDBusObjectPath &trackId, qlonglong pos)
 {
     Q_UNUSED(trackId)
-    m_mpv->setProperty("time-pos", pos/1000/1000);
+    m_mpv->setProperty("time-pos", pos / 1000 / 1000);
 }
 
 void MediaPlayer2Player::OpenUri(const QString &uri)
@@ -121,17 +129,19 @@ QVariantMap MediaPlayer2Player::Metadata()
         return QVariantMap();
     }
     QVariantMap metadata;
-    metadata.insert(QStringLiteral("mpris:length"),
-                    m_mpv->getProperty("duration").toDouble() * 1000 * 1000);
-    metadata.insert(QStringLiteral("mpris:trackid"),
-                    QVariant::fromValue<QDBusObjectPath>(QDBusObjectPath("/org/kde/haruna")));
+    metadata.insert(QStringLiteral("mpris:length"), m_mpv->getProperty("duration").toDouble() * 1000 * 1000);
+    metadata.insert(QStringLiteral("mpris:trackid"), QVariant::fromValue<QDBusObjectPath>(QDBusObjectPath("/org/kde/haruna")));
 
     auto mpvMediaTitle = m_mpv->getProperty("media-title").toString();
     auto mpvFilename = m_mpv->getProperty("filename").toString();
     auto title = mpvMediaTitle.isEmpty() || mpvMediaTitle.isNull() ? mpvFilename : mpvMediaTitle;
     metadata.insert(QStringLiteral("xesam:title"), title);
 
-    QUrl url(m_mpv->getProperty("path").toString());
+    auto path = m_mpv->getProperty("path").toString();
+
+    metadata.insert(QStringLiteral("mpris:artUrl"), getThumbnail(path));
+
+    QUrl url(path);
     url.setScheme("file");
     metadata.insert(QStringLiteral("xesam:url"), url.toString());
 
@@ -143,7 +153,7 @@ double MediaPlayer2Player::Volume()
     if (!m_mpv) {
         return 0;
     }
-    return m_mpv->getProperty("volume").toDouble()/100;
+    return m_mpv->getProperty("volume").toDouble() / 100;
 }
 
 qlonglong MediaPlayer2Player::Position()
@@ -151,10 +161,9 @@ qlonglong MediaPlayer2Player::Position()
     if (!m_mpv) {
         return 0;
     }
-    return m_mpv->getProperty("time-pos").toDouble()*1000*1000;
+    return m_mpv->getProperty("time-pos").toDouble() * 1000 * 1000;
 }
 
-
 bool MediaPlayer2Player::CanGoNext()
 {
     return true;
@@ -198,7 +207,7 @@ void MediaPlayer2Player::setVolume(double vol)
     if (!m_mpv) {
         return;
     }
-    m_mpv->setProperty("volume", vol*100);
+    m_mpv->setProperty("volume", vol * 100);
 }
 
 MpvItem *MediaPlayer2Player::mpv() const
@@ -214,3 +223,49 @@ void MediaPlayer2Player::setMpv(MpvItem *mpv)
     m_mpv = mpv;
     Q_EMIT mpvChanged();
 }
+
+QString MediaPlayer2Player::getThumbnail(const QString &path)
+{
+    QString mimeType = Application::mimeType(QUrl(path));
+    KFileMetaData::ExtractorCollection exCol;
+    QList<KFileMetaData::Extractor *> extractors = exCol.fetchExtractors(mimeType);
+    KFileMetaData::SimpleExtractionResult result(path, mimeType, KFileMetaData::ExtractionResult::ExtractImageData);
+
+    if (extractors.size() > 0) {
+        KFileMetaData::Extractor *ex = extractors.first();
+        ex->extract(&result);
+        QString base64 = QStringLiteral("data:image/png;base64,");
+        auto imageData = result.imageData();
+        if (!imageData.isEmpty()) {
+            // look for image inside the file
+            using namespace KFileMetaData;
+            if (imageData.contains(EmbeddedImageData::FrontCover)) {
+                return base64.append(imageData[EmbeddedImageData::FrontCover].toBase64());
+            } else if (imageData.contains(EmbeddedImageData::BackCover)) {
+                return base64.append(imageData[EmbeddedImageData::BackCover].toBase64());
+            } else if (imageData.contains(EmbeddedImageData::MovieScreenCapture)) {
+                return base64.append(imageData[EmbeddedImageData::MovieScreenCapture].toBase64());
+            } else if (imageData.contains(EmbeddedImageData::Other)) {
+                return base64.append(imageData[EmbeddedImageData::Other].toBase64());
+            } else {
+                return base64.append(imageData[EmbeddedImageData::Unknown].toBase64());
+            }
+        } else {
+            // try to generate QImage from file
+            QImage image = Worker::instance()->frameToImage(path, 250);
+            if (image.isNull()) {
+                // generation failed, use user set default cover
+                return VideoSettings::defaultCover();
+            } else {
+                // generation succeeded
+                QByteArray byteArray;
+                QBuffer buffer(&byteArray);
+                image.save(&buffer, "JPEG");
+                return base64.append(byteArray.toBase64().data());
+            }
+        }
+    }
+    return QString();
+}
+
+#include "moc_mediaplayer2player.cpp"
